<?php
/**************************************************
  Coppermine 1.5.x Plugin - thumbnail_pdf
  *************************************************
  Copyright (c) 2010 S.J.Hoeksma
  *************************************************
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  **************************************************/

if (!defined('IN_COPPERMINE')) die('Not in Coppermine...');

// Add an install action
$thisplugin->add_action('plugin_install', 'thumbnail_pdf_install');

// Add a configure action
$thisplugin->add_action('plugin_configure', 'thumbnail_pdf_configure');

// Add an uninstall action
$thisplugin->add_action('plugin_uninstall','thumbnail_pdf_uninstall');

// Add a filter for displaying the  on display image page
$thisplugin->add_filter('file_data', 'thumbnail_pdf_file_data');


// Add a filter for file upload. This filter is called after the file is successfully uploaded and added to db
$thisplugin->add_filter('add_file_data', 'thumbnail_pdf_add_file_data');

// Add a filter for file delete. When a file is deleted then its thumb should also be deleted
$thisplugin->add_filter('after_delete_file', 'thumbnail_pdf_delete_file');

global $thumbnail_pdf_configs;
$thumbnail_pdf_configs = array('thumbnail_pdf_path','thumbnail_pdf_width');

// Install function
// Inserts the config variables in cpg_config table
function thumbnail_pdf_install() {
    global $CONFIG, $thumbnail_pdf_configs;
    $superCage   = Inspekt::makeSuperCage();
    // Install
    if ($superCage->post->keyExists('thumbnail_pdf_path') && trim($superCage->post->getEscaped('thumbnail_pdf_path'))) {
        foreach ($thumbnail_pdf_configs as $config_name) {
            $config_value = trim($superCage->post->getEscaped($config_name));
            // Run the query to put a config variable for convert path
            if ($config_value) {
                $query = "REPLACE INTO {$CONFIG['TABLE_CONFIG']} SET name = '$config_name', value = '$config_value'";
                cpg_db_query($query);
            }
        }
        return true;

    // Loop again
    } else {

        return 1;
    }
}

// Configure function
// Displays the form
function thumbnail_pdf_configure() {
    global $CONFIG;
    
    if (!isset($CONFIG['thumbnail_pdf_width'])) {
        $CONFIG['thumbnail_pdf_width'] = 300;
    }
        
    
    echo <<< EOT
    <form name="cpgform" id="cpgform" action="{$_SERVER['REQUEST_URI']}" method="post">
            <table border="0" cellspacing="0" cellpadding="0" width="100%">
              <tr>
                <td class="tableb" align="right" width="250">
                  Absolute path to convert binary:
                </td>
                <td class="tableb">
                  <input type="text" name="thumbnail_pdf_path" class="textinput" style="width:50%;" value="{$CONFIG['thumbnail_pdf_path']}" />
                </td>
              </tr>
		<tr>
		      <td class="tableb" align="right" width="250">
			Width of the PDF Image:
		      </td>
		      <td class="tableb">
			<input type="text" name="thumbnail_pdf_width" class="textinput" size="3" value="{$CONFIG['thumbnail_pdf_width']}" />
		      </td>
		    </tr>
              <tr>
                <td class="tablef" colspan="2">
                  <input type="submit" value="Go!" class="button" />
                </td>
              </tr>
            </table>
    </form>
EOT;
}

/**
 * Function called when the plugin is uninstalled
 * 
 * We will delete the plugin related config values from db
 */
function thumbnail_pdf_uninstall() {
    global $CONFIG, $thumbnail_pdf_configs;
    // Delete the plugin related config values
    foreach ($thumbnail_pdf_configs as $name) {
        $f = cpg_db_query("DELETE FROM {$CONFIG['TABLE_CONFIG']} WHERE `name` = '$name'");
    }
    return true;
}

function is_pdf(&$file) {
  return pathinfo($file, PATHINFO_EXTENSION) == 'pdf';
}


/**
 * This function is called after the file is successfully uploaded and added to db
 * This function will create the thumb
 */
function thumbnail_pdf_add_file_data($pic_data) {
    global $CONFIG;
    $filepath = $pic_data['filepath'];
    $filename = $pic_data['filename'];
    $image = $CONFIG['fullpath'] . $filepath . $filename;
    if (is_pdf($image)) { // is PDF
        preg_match("/(.+)\.(.*?)\Z/", $filename, $matches);
        $thumb = $CONFIG['fullpath'] . $filepath . $CONFIG['thumb_pfx'] . $matches[1].".jpg";
        $normal = $CONFIG['fullpath'] . $filepath . $CONFIG['normal_pfx'] . $matches[1].".jpg";
        $pdfThumb = create_pdf_image($image);
        $imagesize = getimagesize($pdfThumb);
        if ($pdfThumb) {
            if (!resize_image($pdfThumb, $thumb, $CONFIG['thumb_width'], $CONFIG['thumb_method'], $CONFIG['thumb_use'])) {
                // If resize fails then we are not doing anything as the file is already uploaded.
            } else {   
                // Also create a normal size image to show as a preview
                if (resize_image($pdfThumb, $normal, $CONFIG['thumbnail_pdf_width'], $CONFIG['thumb_method'], $CONFIG['thumb_use'])) {
                    // Add the filesize of normal to the total filesize
                    $pic_data['total_filesize'] += filesize($normal);
                }
                $pic_data['pwidth'] = $imagesize[0];
                $pic_data['pheight'] = $imagesize[1];
                $pic_data['total_filesize'] += filesize($thumb);
            }
           @unlink($pdfThumb);
        } else {
            // If thumb is not created then proceed and don't stop here. We are not stopping as the file is already uploaded.
       }

    }

    return $pic_data;
}

/**
 * Function to return html required for displaying the normal image
 */
function thumbnail_pdf_file_data($pic_data) {
    global $CONFIG;

    $file = $CONFIG['fullpath'] . $pic_data['filepath'] . $pic_data['filename'];
    preg_match("/(.+)\.(.*?)\Z/", $pic_data['filename'], $matches);
    $normal = $CONFIG['fullpath'] . $pic_data['filepath'] . $CONFIG['normal_pfx'] . $matches[1].".jpg";
    if (is_pdf($file)  && file_exists($normal)) { // is PDF    
      $pic_data['html'] = <<<EOT
       <a href="$file" target="_blank" class="document_link"><img src="$normal" border="0" class="image" /></a><br />
EOT;
    }
    return $pic_data;
}

/**
 * Function to delete thumb and normal image
 */
function thumbnail_pdf_delete_file($pic) {
    global $CONFIG;
    $file = $CONFIG['fullpath'] . $pic['filepath'] . $pic['filename'];
    preg_match("/(.+)\.(.*?)\Z/", $pic['filename'], $matches);
    $thumb = $CONFIG['fullpath'] . $pic['filepath'] . $CONFIG['thumb_pfx'] . $matches[1].".jpg";
    $normal = $CONFIG['fullpath'] . $pic['filepath'] . $CONFIG['normal_pfx'] . $matches[1].".jpg";    
    @unlink($thumb);
    @unlink($normal);
}

/**
 * Function to create pdf image using convert
 */
function create_pdf_image($src_file) {
    global $CONFIG;

    $dest_file = $CONFIG['fullpath'] . "edit/" . md5(uniqid(time())).".jpg";

    if (preg_match("#[A-Z]:|\\\\#Ai", __FILE__)) {
        // get the basedir, remove '/include'
        $cur_dir = substr(dirname(__FILE__), 0, -8);
        $src_file = '"' . $cur_dir . '\\' . strtr($src_file, '/', '\\') . '[0]"';
        $ff_dest_file = '"' . $cur_dir . '\\' . strtr($dest_file, '/', '\\') . '"';
    } else {
        $src_file = escapeshellarg($src_file."[0]");
        $ff_dest_file = escapeshellarg($dest_file);
    }

    $output = array();

    // Command to create base image first page of pdf
    if (eregi("win", $_ENV['OS'])) {
        $cmd = "\"".str_replace("\\","/", $ONFIG['thumbnail_pdf_path'])."\" \"".str_replace("\\","/" ,$src_file )." -colorspace RGB -geometry 1024 ".str_replace("\\","/" ,$ff_dest_file);
        exec ("\"$cmd\"", $output, $retval);
    } else {
        $cmd = "{$CONFIG['thumbnail_pdf_path']} {$src_file} -colorspace RGB -geometry 2048 $ff_dest_file";
        exec ($cmd, $output, $retval);
    }

    if ($retval) {
        @unlink($dest_file);
        return false;
    }

    $return = $dest_file;
    @chmod($return, octdec($CONFIG['default_file_mode'])); //silence the output in case chmod is disabled
    return $return;
}
?>
